/* ReceiveProgressDialog.c generated by valac 0.54.3, the Vala compiler
 * generated from ReceiveProgressDialog.vala, do not modify */

/*
 *  Copyright (C) 2009-2010 Michael J. Chudobiak.
 *
 *  This file is part of moserial.
 *
 *  moserial is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  moserial is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with moserial.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib-object.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <config.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <string.h>

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define MOSERIAL_TYPE_RECEIVE_PROGRESS_DIALOG (moserial_receive_progress_dialog_get_type ())
#define MOSERIAL_RECEIVE_PROGRESS_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MOSERIAL_TYPE_RECEIVE_PROGRESS_DIALOG, moserialReceiveProgressDialog))
#define MOSERIAL_RECEIVE_PROGRESS_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MOSERIAL_TYPE_RECEIVE_PROGRESS_DIALOG, moserialReceiveProgressDialogClass))
#define MOSERIAL_IS_RECEIVE_PROGRESS_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MOSERIAL_TYPE_RECEIVE_PROGRESS_DIALOG))
#define MOSERIAL_IS_RECEIVE_PROGRESS_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MOSERIAL_TYPE_RECEIVE_PROGRESS_DIALOG))
#define MOSERIAL_RECEIVE_PROGRESS_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MOSERIAL_TYPE_RECEIVE_PROGRESS_DIALOG, moserialReceiveProgressDialogClass))

typedef struct _moserialReceiveProgressDialog moserialReceiveProgressDialog;
typedef struct _moserialReceiveProgressDialogClass moserialReceiveProgressDialogClass;
typedef struct _moserialReceiveProgressDialogPrivate moserialReceiveProgressDialogPrivate;
enum  {
	MOSERIAL_RECEIVE_PROGRESS_DIALOG_0_PROPERTY,
	MOSERIAL_RECEIVE_PROGRESS_DIALOG_NUM_PROPERTIES
};
static GParamSpec* moserial_receive_progress_dialog_properties[MOSERIAL_RECEIVE_PROGRESS_DIALOG_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
enum  {
	MOSERIAL_RECEIVE_PROGRESS_DIALOG_TRANSFER_CANCELED_SIGNAL,
	MOSERIAL_RECEIVE_PROGRESS_DIALOG_NUM_SIGNALS
};
static guint moserial_receive_progress_dialog_signals[MOSERIAL_RECEIVE_PROGRESS_DIALOG_NUM_SIGNALS] = {0};

struct _moserialReceiveProgressDialog {
	GObject parent_instance;
	moserialReceiveProgressDialogPrivate * priv;
};

struct _moserialReceiveProgressDialogClass {
	GObjectClass parent_class;
};

struct _moserialReceiveProgressDialogPrivate {
	GtkDialog* dialog;
	GtkButton* cancelButton;
	GtkLabel* status;
	GtkProgressBar* progressBar;
};

static gint moserialReceiveProgressDialog_private_offset;
static gpointer moserial_receive_progress_dialog_parent_class = NULL;

VALA_EXTERN GType moserial_receive_progress_dialog_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (moserialReceiveProgressDialog, g_object_unref)
VALA_EXTERN moserialReceiveProgressDialog* moserial_receive_progress_dialog_new (GtkWindow* parent);
VALA_EXTERN moserialReceiveProgressDialog* moserial_receive_progress_dialog_construct (GType object_type,
                                                                           GtkWindow* parent);
VALA_EXTERN void moserial_receive_progress_dialog_cancel (moserialReceiveProgressDialog* self,
                                              GObject* o);
static void _moserial_receive_progress_dialog_cancel_gtk_button_clicked (GtkButton* _sender,
                                                                  gpointer self);
VALA_EXTERN gboolean moserial_receive_progress_dialog_hide (moserialReceiveProgressDialog* self);
static gboolean _moserial_receive_progress_dialog_hide_gtk_widget_delete_event (GtkWidget* _sender,
                                                                         GdkEventAny* event,
                                                                         gpointer self);
VALA_EXTERN void moserial_receive_progress_dialog_show (moserialReceiveProgressDialog* self);
VALA_EXTERN void moserial_receive_progress_dialog_updateStatus (moserialReceiveProgressDialog* self,
                                                    GObject* o,
                                                    const gchar* newStatus);
static void moserial_receive_progress_dialog_finalize (GObject * obj);
static GType moserial_receive_progress_dialog_get_type_once (void);

static inline gpointer
moserial_receive_progress_dialog_get_instance_private (moserialReceiveProgressDialog* self)
{
	return G_STRUCT_MEMBER_P (self, moserialReceiveProgressDialog_private_offset);
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static void
_moserial_receive_progress_dialog_cancel_gtk_button_clicked (GtkButton* _sender,
                                                             gpointer self)
{
	moserial_receive_progress_dialog_cancel ((moserialReceiveProgressDialog*) self, _sender);
}

static gboolean
_moserial_receive_progress_dialog_hide_gtk_widget_delete_event (GtkWidget* _sender,
                                                                GdkEventAny* event,
                                                                gpointer self)
{
	gboolean result;
	result = moserial_receive_progress_dialog_hide ((moserialReceiveProgressDialog*) self);
	return result;
}

moserialReceiveProgressDialog*
moserial_receive_progress_dialog_construct (GType object_type,
                                            GtkWindow* parent)
{
	moserialReceiveProgressDialog * self = NULL;
	GtkBuilder* builder = NULL;
	GtkBuilder* _tmp0_;
	GObject* _tmp1_;
	GtkDialog* _tmp2_;
	GtkDialog* _tmp3_;
	GObject* _tmp4_;
	GtkButton* _tmp5_;
	GtkButton* _tmp6_;
	GObject* _tmp7_;
	GtkLabel* _tmp8_;
	GObject* _tmp9_;
	GtkProgressBar* _tmp10_;
	GtkDialog* _tmp11_;
	g_return_val_if_fail (parent != NULL, NULL);
	self = (moserialReceiveProgressDialog*) g_object_new (object_type, NULL);
	_tmp0_ = gtk_builder_new_from_resource (UIROOT "receive_progress.ui");
	builder = _tmp0_;
	_tmp1_ = gtk_builder_get_object (builder, "receive_progress_dialog");
	_tmp2_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp1_, gtk_dialog_get_type (), GtkDialog));
	_g_object_unref0 (self->priv->dialog);
	self->priv->dialog = _tmp2_;
	_tmp3_ = self->priv->dialog;
	gtk_window_set_transient_for ((GtkWindow*) _tmp3_, parent);
	_tmp4_ = gtk_builder_get_object (builder, "receive_progress_cancel");
	_tmp5_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp4_, gtk_button_get_type (), GtkButton));
	_g_object_unref0 (self->priv->cancelButton);
	self->priv->cancelButton = _tmp5_;
	_tmp6_ = self->priv->cancelButton;
	g_signal_connect_object (_tmp6_, "clicked", (GCallback) _moserial_receive_progress_dialog_cancel_gtk_button_clicked, self, 0);
	_tmp7_ = gtk_builder_get_object (builder, "receive_statusbox");
	_tmp8_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp7_, gtk_label_get_type (), GtkLabel));
	_g_object_unref0 (self->priv->status);
	self->priv->status = _tmp8_;
	_tmp9_ = gtk_builder_get_object (builder, "receive_progressbar");
	_tmp10_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_CAST (_tmp9_, gtk_progress_bar_get_type (), GtkProgressBar));
	_g_object_unref0 (self->priv->progressBar);
	self->priv->progressBar = _tmp10_;
	_tmp11_ = self->priv->dialog;
	g_signal_connect_object ((GtkWidget*) _tmp11_, "delete-event", (GCallback) _moserial_receive_progress_dialog_hide_gtk_widget_delete_event, self, 0);
	_g_object_unref0 (builder);
	return self;
}

moserialReceiveProgressDialog*
moserial_receive_progress_dialog_new (GtkWindow* parent)
{
	return moserial_receive_progress_dialog_construct (MOSERIAL_TYPE_RECEIVE_PROGRESS_DIALOG, parent);
}

void
moserial_receive_progress_dialog_show (moserialReceiveProgressDialog* self)
{
	GtkDialog* _tmp0_;
	GtkLabel* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->dialog;
	gtk_widget_show_all ((GtkWidget*) _tmp0_);
	_tmp1_ = self->priv->status;
	gtk_label_set_text (_tmp1_, "");
}

gboolean
moserial_receive_progress_dialog_hide (moserialReceiveProgressDialog* self)
{
	GtkDialog* _tmp0_;
	gboolean result = FALSE;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = self->priv->dialog;
	gtk_widget_hide ((GtkWidget*) _tmp0_);
	result = TRUE;
	return result;
}

void
moserial_receive_progress_dialog_updateStatus (moserialReceiveProgressDialog* self,
                                               GObject* o,
                                               const gchar* newStatus)
{
	GtkLabel* _tmp0_;
	GtkProgressBar* _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (o != NULL);
	g_return_if_fail (newStatus != NULL);
	_tmp0_ = self->priv->status;
	gtk_label_set_text (_tmp0_, newStatus);
	_tmp1_ = self->priv->progressBar;
	gtk_progress_bar_pulse (_tmp1_);
}

void
moserial_receive_progress_dialog_cancel (moserialReceiveProgressDialog* self,
                                         GObject* o)
{
	g_return_if_fail (self != NULL);
	g_return_if_fail (o != NULL);
	g_signal_emit (self, moserial_receive_progress_dialog_signals[MOSERIAL_RECEIVE_PROGRESS_DIALOG_TRANSFER_CANCELED_SIGNAL], 0);
}

static void
moserial_receive_progress_dialog_class_init (moserialReceiveProgressDialogClass * klass,
                                             gpointer klass_data)
{
	moserial_receive_progress_dialog_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &moserialReceiveProgressDialog_private_offset);
	G_OBJECT_CLASS (klass)->finalize = moserial_receive_progress_dialog_finalize;
	moserial_receive_progress_dialog_signals[MOSERIAL_RECEIVE_PROGRESS_DIALOG_TRANSFER_CANCELED_SIGNAL] = g_signal_new ("transfer-canceled", MOSERIAL_TYPE_RECEIVE_PROGRESS_DIALOG, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}

static void
moserial_receive_progress_dialog_instance_init (moserialReceiveProgressDialog * self,
                                                gpointer klass)
{
	self->priv = moserial_receive_progress_dialog_get_instance_private (self);
}

static void
moserial_receive_progress_dialog_finalize (GObject * obj)
{
	moserialReceiveProgressDialog * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MOSERIAL_TYPE_RECEIVE_PROGRESS_DIALOG, moserialReceiveProgressDialog);
	_g_object_unref0 (self->priv->dialog);
	_g_object_unref0 (self->priv->cancelButton);
	_g_object_unref0 (self->priv->status);
	_g_object_unref0 (self->priv->progressBar);
	G_OBJECT_CLASS (moserial_receive_progress_dialog_parent_class)->finalize (obj);
}

static GType
moserial_receive_progress_dialog_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (moserialReceiveProgressDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) moserial_receive_progress_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (moserialReceiveProgressDialog), 0, (GInstanceInitFunc) moserial_receive_progress_dialog_instance_init, NULL };
	GType moserial_receive_progress_dialog_type_id;
	moserial_receive_progress_dialog_type_id = g_type_register_static (G_TYPE_OBJECT, "moserialReceiveProgressDialog", &g_define_type_info, 0);
	moserialReceiveProgressDialog_private_offset = g_type_add_instance_private (moserial_receive_progress_dialog_type_id, sizeof (moserialReceiveProgressDialogPrivate));
	return moserial_receive_progress_dialog_type_id;
}

GType
moserial_receive_progress_dialog_get_type (void)
{
	static volatile gsize moserial_receive_progress_dialog_type_id__volatile = 0;
	if (g_once_init_enter (&moserial_receive_progress_dialog_type_id__volatile)) {
		GType moserial_receive_progress_dialog_type_id;
		moserial_receive_progress_dialog_type_id = moserial_receive_progress_dialog_get_type_once ();
		g_once_init_leave (&moserial_receive_progress_dialog_type_id__volatile, moserial_receive_progress_dialog_type_id);
	}
	return moserial_receive_progress_dialog_type_id__volatile;
}

